#! /bin/sh
#
# Copyright by The HDF Group.
# All rights reserved.
#
# Tests for the h5edit tool

TESTNAME=h5edit
EXIT_SUCCESS=0
EXIT_FAILURE=1

H5EDIT=h5edit               # The tool name
H5EDIT_BIN=`pwd`/../src/$H5EDIT    # The path of the h5edit binary

DIFF='diff -c'
NLINES=20		    # Max. lines of output to display if test fails


TESTFILE=tmptestfile.h5	    # name of temporary HDF5 testfile
nerrors=0
verbose=yes
h5haveexitcode=yes	    # default is yes

# The build (current) directory might be different than the source directory.
if test -z "$srcdir"; then
    srcdir=.
fi
test -d ./testfiles || mkdir ./testfiles

# RUNSERIAL is used. Check if it can return exit code from executable correctly.
if [ -n "$RUNSERIAL_NOEXITCODE" ]; then
    echo "***Warning*** Serial Exit Code is not passed back to shell correctly."
    echo "***Warning*** Exit code checking is skipped."
    h5haveexitcode=no
fi

# Print a line-line message left justified in a field of 70 characters
# beginning with the word "Testing".
TESTING() {
    SPACES="                                                               "
    echo "Testing $* $SPACES" |cut -c1-70 |tr -d '\012'
}

# Copy datafile ($1) from testfile directory ($srcdir/testfiles) to
# testfile ($2) for testing. If $2 is not given, assume current directory and
# use same datafile name.
COPYFILE() {
    if [ -z "$1" ]; then
	echo "COPYFILE: datafile argument missing"
	exit $EXIT_FAILURE
    fi
    if [ ! -r $1 ]; then
	echo "COPYFILE: datafile not readable"
	exit $EXIT_FAILURE
    fi
    if [ -n "$2" ]; then
	# copy to filename $2
	cp $1 $2
    else
	# copy to current directory using same name
	cp $1 .
    fi
    return 0
}


# Run a test and print PASS or *FAIL*. For now, if h5edit can complete
# with exit status 0, consider it pass. If a test fails then increment
# the `nerrors' global variable and (if $verbose is set) display up to $NLINES
# lines of the actual output from the test.  The actual output is not
# removed if $HDF5_NOCLEANUP has a non-zero value.
# Arguments:
# $1 -- expected output filename to use, found in $srcdir/testfiles directory
# $2 and on -- argument for the h5edit tool
# If $1 is nocheck, then do not check the output.
H5EDITTEST()
{
    if [ $1 = nocheck ]; then
	nocheckfile=yes
    else
    	nocheckfile=
    fi
    expect="$srcdir/testfiles/$1"
    h5ref="$srcdir/testfiles/`basename $1 .edit`.ref"
    result="./testfiles/`basename $1 .edit`.result"
    actual="./testfiles/`basename $1 .edit`.out"
    actual_err="./testfiles/`basename $1 .edit`.err"
    shift
    retvalexpect=$1
    shift
    data_correct=yes

#    if test "$nocheckfile" != ""; then
#        echo "nocheckfile is $nocheckfile."
#    fi

    # Run test.
    # Stderr is included in stdout so that the diff can detect
    # any unexpected output from that stream too.
    TESTING $H5EDIT $@
    (
	cd testfiles
        $RUNSERIAL $H5EDIT_BIN "$@"
    ) >$actual 2>$actual_err 
    exitcode=$?
    cat $actual_err >> $actual 

    if [ $h5haveexitcode = 'yes' -a $exitcode -ne $retvalexpect ]; then
	echo "*FAILED*"
	nerrors="`expr $nerrors + 1`"
	if [ yes = "$verbose" ]; then
	    echo "test returned with exit code $exitcode"
	    echo "test output: (up to $NLINES lines)"
	    head -$NLINES $actual_err
	    head -$NLINES $actual
	    echo "***end of test output***"
	    echo ""
	fi
    # not checking against any expected file
    elif test "$nocheckfile" != ""; then
        echo " PASSED"
    # Otherwise, compare the output to the expected file
    #elif test -f $expect && test -f $actual && $DIFF $expect $actual; then
    else
        # if there's a datafile, dump it to a text file for diff against the expected file.
        if [ -f testfiles/$TESTFILE ]; then
            h5dump testfiles/$TESTFILE > $result
            $DIFF $h5ref $result > /dev/null 2>&1
            if [ "$?" != 0 ]; then
                data_correct=no
            fi
        fi
        if test -f $expect && test -f $actual; then
            $DIFF $expect $actual > /dev/null 2>&1
            if [ "$?" = "0" ]; then
                if [ "$data_correct" = "no" ]; then
                        echo "*FAILED*"
        	        echo "    Expected result differs from actual result"
        	        nerrors="`expr $nerrors + 1`"
    	            test yes = "$verbose" && $DIFF $h5ref $result |sed 's/^/    /' \
                                          && echo ""
                else
                    echo " PASSED"
                    if [ -f testfiles/$TESTFILE ]; then
                       rm testfiles/$TESTFILE
                    fi
                fi
            else
                echo " *FAILED*"
                echo "    Expected output differs from actual output"
                nerrors="`expr $nerrors + 1`"
                test yes = "$verbose" && $DIFF $expect $actual |sed 's/^/    /'
                if [ "$data_correct" = "no" ]; then
    	        echo "    Expected result differs from actual result"
    	        test yes = "$verbose" && $DIFF $h5ref $result |sed 's/^/    /' \
                                      && echo ""
                fi           
            fi
        else
            echo " *FAILED*"
            test yes = "$verbose" && echo "$expect not found" && echo ""
            nerrors="`expr $nerrors + 1`"
        fi
    fi

    # Clean up output file
    if test -z "$HDF5_NOCLEANUP"; then
        rm -f testfiles/$TESTFILE
	rm -f $actual $actual_err $result
    fi
}

##############################################################################
##############################################################################
###			  T H E   T E S T S                                ###
##############################################################################
##############################################################################

# test the help syntax - valid
H5EDITTEST help-1.edit 0 "-h"
H5EDITTEST help-1.edit 0 "--help"

# test version information
H5EDITTEST version.edit 0 "--version"

# test the help syntax - incomplete 
H5EDITTEST missing-command.edit 1 ""
H5EDITTEST missing-command-arg.edit 1 "-c"
H5EDITTEST missing-command-arg.edit 1 "--command"
H5EDITTEST missing-command-file-arg.edit 1 "--command-file"
H5EDITTEST missing-command.edit 1 "--dryrun"
H5EDITTEST unknownoption-1.edit 1 "-help"

# test the help syntax - invalid
H5EDITTEST unknownoption-2.edit  1 "-x"
H5EDITTEST unknownoption-3.edit  1 "--bogus"
H5EDITTEST missing-command.edit  1 "%@bogus_command@%"
H5EDITTEST missing-command.edit 1 '`pwd`'

# non-existent command file
H5EDITTEST bad-command-file.edit 1 --command-file ./no_such_file.h5

# test simple legal commands
# Copy the datafile and test
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST delete-atts12-dset111.edit 0 \
    -c "DELETE /g1/g1.1/dset1.1.1 attr1; \
        DELETE DATASET /g1/g1.1/dset1.1.1 attr2;" \
    $TESTFILE

# Repeat with single combined target-object/attribute syntax.
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST delete-atts12-dset111.edit 0 \
    -c "DELETE /g1/g1.1/dset1.1.1/attr1; \
        DELETE DATASET /g1/g1.1/dset1.1.1/attr2;" \
    $TESTFILE

# Test DELETE from Group
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST delete-atts12-grrt.edit 0 \
    -c "DELETE GROUP / attr1; DELETE GROUP / attr2;" $TESTFILE

# Repeat with single combined target-object/attribute syntax.
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST delete-atts12-grrt.edit 0 \
    -c "DELETE /attr1; DELETE /attr2;" $TESTFILE

# Test CREATE Command
# Test CREATE SCALAR integer attibutes
# This also checks the validity of the keywords DATATYPE, DATASPACE, DATA.
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-newatts-I3264BE.edit 0 \
    -c "CREATE /g1/g1.1/dset1.1.1 newattrI32BE {DATATYPE H5T_STD_I32BE DATASPACE SCALAR DATA {32}} ; \
	CREATE /g1/g1.1/dset1.1.1 newattrI64BE {DATATYPE H5T_STD_I64BE DATASPACE SCALAR DATA {64}} ;" \
    $TESTFILE

# Test CREATE SCALAR float attibutes
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-newatts-F32BE64LE.edit 0 \
    -c "CREATE /g1/g1.1/dset1.1.1 newattrF32BE { H5T_IEEE_F32BE SCALAR {32.32}} ; \
	CREATE /g1/g1.1/dset1.1.1 newattrF32LE { H5T_IEEE_F32LE SCALAR {64.64}} ;" \
    $TESTFILE

# Repeat with single combined target-object/attribute syntax.
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-newatts-F32BE64LE.edit 0 \
    -c "CREATE /g1/g1.1/dset1.1.1/newattrF32BE { H5T_IEEE_F32BE SCALAR {32.32}} ; \
	CREATE /g1/g1.1/dset1.1.1/newattrF32LE { H5T_IEEE_F32LE SCALAR {64.64}} ;" \
    $TESTFILE

# Test CREATE Simple Atributes for integer and float attributes
# This also checks the validity of the keyword SIMPLE.
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-newatts-F64LEI32BE.edit 0  -c "CREATE /g1/g1.1/dset1.1.1 newattrI32BE {H5T_STD_I32BE SIMPLE ( 1, 2, 3) {11, 12, 21, 22, 31,32}}; \
CREATE /g1/g1.1/dset1.1.1 newattrF64LE {H5T_IEEE_F64LE SIMPLE (2, 2, 2 ) {0.1E1, 0.2E1, -1.1e2, -1.2e2, 2.1E-3, 2.2E-3, -3.1E-3, -3.2E-2}} ;" $TESTFILE

# Test Command file containing the same CREATE commands as above.
# Should have the same result as above.
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-newatts-F64LEI32BE.edit 0 --command-file ../$srcdir/testfiles/t_comm_file1 $TESTFILE

# Test CREATE SCALAR dataspace attributes with integer datatypes
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-intattsscalar.edit 0 -c "CREATE GROUP / newattrI8LE {H5T_STD_I8LE SCALAR {8}} ; CREATE GROUP /g1 newattrI8BE {H5T_STD_I8BE SCALAR {8}} ; CREATE GROUP /g1/g1.1 newattrI16LE {H5T_STD_I16LE SCALAR {16}} ; CREATE /g1/g1.1/dset1.1.1 newattrI16BE {H5T_STD_I16BE SCALAR {16}} ; CREATE /g1/g1.1/dset1.1.2 newattrI32LE {H5T_STD_I32LE SCALAR {32}} ; CREATE GROUP /g1/g1.2 newattrI32BE {H5T_STD_I32BE SCALAR {32}} ; CREATE GROUP /g1/g1.2/g1.2.1 newattrI64LE {H5T_STD_I64LE SCALAR {64}} ; CREATE /g2/dset2.2 newattrI64BE {H5T_STD_I64BE SCALAR {64}} ; " $TESTFILE

# Test CREATE SIMPLE dataspace attributes with integer datatypes
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-intattssimple.edit 0 -c "CREATE GROUP /g1/g1.1 newattrI8LE {H5T_STD_I8LE ( 1, 2, 3) {2, 4, 8, 16, 32, 64}} ; CREATE /g1/g1.1/dset1.1.1 newattrI8BE {H5T_STD_I8BE (2, 2, 2 ) {-2, 0, 2, 4, 8, 16, 32, 64}} ; CREATE /g1/g1.1/dset1.1.2 newattrI16LE {H5T_STD_I16LE ( 1, 2, 3)  {64, 128, 256, 512, 1024, 2048}} ; CREATE GROUP /g1/g1.2 newattrI16BE {H5T_STD_I16BE (2, 2, 2 )  {64, 128, 256, 512, 1024, 2048, 4096, 8192}} ; CREATE GROUP /g1/g1.2/g1.2.1 newattrI32LE {H5T_STD_I32LE ( 1, 2, 3)  {2048, 4096, 8192, 16384, 32768, 65536}} ; CREATE GROUP /g2 newattrI32BE {H5T_STD_I32BE (2, 2, 2 )  {2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144}} ;CREATE /g2/dset2.1 newattrI64LE {H5T_STD_I64LE ( 1, 2, 3)  {65536, 131072, 262144, 524288, 1048576, 2097152}} ; CREATE /g2/dset2.2 newattrI64BE {H5T_STD_I64BE (2, 2, 2 )  {65536, 131072, 262144, 524288, 1048576, 2097152, 4194304, 8388608}} ; " $TESTFILE

# Test CREATE SCALAR dataspace attributes with unsigned integer datatypes
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-unsignedattsscalar.edit 0 -c "CREATE GROUP / newattrU8LE {H5T_STD_U8LE SCALAR {8}} ; CREATE GROUP /g1 newattrU8BE {H5T_STD_U8BE SCALAR {8}} ; CREATE GROUP /g1/g1.1 newattrU16LE {H5T_STD_U16LE SCALAR {16}} ; CREATE /g1/g1.1/dset1.1.1 newattrU16BE {H5T_STD_U16BE SCALAR {16}} ; CREATE /g1/g1.1/dset1.1.2 newattrU32LE {H5T_STD_U32LE SCALAR {32}} ; CREATE GROUP /g1/g1.2 newattrU32BE {H5T_STD_U32BE SCALAR {32}} ;CREATE GROUP /g1/g1.2/g1.2.1 newattrU64LE {H5T_STD_U64LE SCALAR {64}} ; CREATE /g2/dset2.2 newattrU64BE {H5T_STD_U64BE SCALAR {64}} ; " $TESTFILE

# Test CREATE unsigned integer datatypes
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-unsignedattssimple.edit 0 -c "CREATE GROUP /g1/g1.1 newattrU8LE {H5T_STD_U8LE ( 1, 2, 3)  {2, 4, 8, 16, 32, 64}} ; CREATE /g1/g1.1/dset1.1.1 newattrU8BE {H5T_STD_U8BE (2, 2, 2 )  {0, 2, 4, 8, 16, 32, 64, 128}} ; CREATE /g1/g1.1/dset1.1.2 newattrU16LE {H5T_STD_U16LE ( 1, 2, 3)  {64, 128, 256, 512, 1024, 2048}} ; CREATE GROUP /g1/g1.2 newattrU16BE {H5T_STD_U16BE (2, 2, 2 )  {64, 128, 256, 512, 1024, 2048, 4096, 8192}} ; CREATE GROUP /g1/g1.2/g1.2.1 newattrU32LE {H5T_STD_U32LE ( 1, 2, 3)  {2048, 4096, 8192, 16384, 32768, 65536}} ; CREATE GROUP /g2 newattrU32BE {H5T_STD_U32BE (2, 2, 2 )  {2048, 4096, 8192, 16384, 32768, 65536, 131072, 262144}} ;CREATE /g2/dset2.1 newattrU64LE {H5T_STD_U64LE ( 1, 2, 3)  {65536, 131072, 262144, 524288, 1048576, 2097152}} ; CREATE /g2/dset2.2 newattrU64BE {H5T_STD_U64BE (2, 2, 2 )  {65536, 131072, 262144, 524288, 1048576, 2097152, 4194304, 8388608}} ; " $TESTFILE

# Test CREATE SCALAR dataspace attributes with float datatypes
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-floatattsscalar.edit 0 -c "CREATE /g1/g1.1/dset1.1.1 newattrF32BE {H5T_IEEE_F32BE SCALAR {32.32}} ; CREATE GROUP /g1/g1.2 newattrF64BE {H5T_IEEE_F64BE SCALAR {64.64}} ; CREATE /g1/g1.1/dset1.1.1 newattrF32LE {H5T_IEEE_F32LE SCALAR {32.32}} ; CREATE GROUP /g1/g1.2 newattrF64LE {H5T_IEEE_F64LE SCALAR {64.64}} ;" $TESTFILE

# Test CREATE dataspace attributes with float datatypes
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-floatattssimple.edit 0  -c "CREATE /g1/g1.1/dset1.1.1 newattrF32BE {H5T_IEEE_F32BE ( 1, 2, 3)  {11.1111, 12.1212, 21.2121, 22.2222, 31.3131, 32.3232}}; CREATE GROUP /g1/g1.2 newattrF32LE {H5T_IEEE_F32LE ( 1, 2, 3)  {11.1111, 12.1212, 21.2121, 22.2222, 31.3131, 32.3232}}; CREATE /g1/g1.1/dset1.1.1 newattrF64BE {H5T_IEEE_F64BE (2, 2, 2 )  {0.1E1, 0.2E1, -1.1e2, -1.2e2, 2.1E-3, 2.2E-3, -3.1E-3, -3.2E-2}} ; CREATE GROUP /g1/g1.2 newattrF64LE {H5T_IEEE_F64LE (2, 2, 2 )  {0.1E1, 0.2E1, -1.1e2, -1.2e2, 2.1E-3, 2.2E-3, -3.1E-3, -3.2E-2}} ;" $TESTFILE

# Test CREATE SCALAR and STRING type attributes
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-strings.edit 0 -c "CREATE /g1/g1.1/dset1.1.1 newattrScalarString {H5T_STRING { STRSIZE 15 } SCALAR  {'scalar string'}}; CREATE /g1/g1.1/dset1.1.1 newattrArrayString {H5T_STRING { STRSIZE 10 } ( 3 )  {'a', 'string', 'array'}}; " $TESTFILE

# The following commands should fail.  Currently they return 0 with error messages.  If they should return
# 1, the second argument to H5EDITTEST will need to be changed when they are fixed.
# Test CREATE Command for existing attribute
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-existingattr.edit 0 -c "CREATE GROUP / attr1 {H5T_STD_I8BE (1, 10) { 9, 8, 7, 6, 5, 4, 3, 2, 1, 0 }} ;" $TESTFILE

# Test CREATE command for non-exiting object
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST create-attr-missingdset.edit 0 -c "CREATE /g1/bogusdset1.1 attr1 {H5T_STD_I8BE (1, 10) { 9, 8, 7, 6, 5, 4, 3, 2, 1, 0 }} ;" $TESTFILE

# Test DELETE command for non-existing attribute
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST delete-bogusattr.edit 0 -c "DELETE /g1/g1.1/dset1.1.1 bogusattr;" $TESTFILE

# Test DELETE command for non-existing object
COPYFILE $srcdir/testfiles/datafile1.h5 testfiles/$TESTFILE
H5EDITTEST delete-bogusgroup.edit 0 -c "DELETE GROUP /g1/g1.1000 attr1;" $TESTFILE
 

# Post results and exit.
if test $nerrors -eq 0 ; then
    echo "All $TESTNAME tests passed."
    exit $EXIT_SUCCESS
else
    echo "$TESTNAME tests failed with $nerrors errors."
    exit $EXIT_FAILURE
fi
